﻿using System;
using System.IO;
using System.Threading;
using System.Web;
using System.Web.SessionState;

namespace Core.Module.HttpFilter
{
    public class HttpFileterStream : Stream, IRequiresSessionState
    {
        private HttpApplication app;//HttpApplication对象
        private Stream output;//Http输出流
        private HttpFilterFactory filterHandler;//过滤器处理
        private FilterContext filterContext;//HttpFilterContext上下文信息

        public HttpFileterStream(HttpApplication app, HttpFilterFactory filterHandler, FilterContext filterContext)
        {
            this.app = app;
            this.filterHandler = filterHandler;
            this.output = app.Response.Filter;
            this.filterContext = filterContext;
        }

        public int Filter(byte[] buffer, int offset, int count)
        {
            var response = app.Response;
            var request = app.Request;

            //判断参数是否合法
            if (null == buffer || 0 > buffer.Length || 0 > offset || 0 > count)
            {
                FilterLog.Log.Debug("Filter 参数无效");
                return 1;
            }

            //仅拦截状态码为 200 OK
            if (200 != response.StatusCode)
            {
                FilterLog.Log.DebugFormat("状态码非200，为:{0}", response.StatusCode);
                return 2;
            }

            //记录Http输出内容体
            filterContext.OutputBody = response.ContentEncoding.GetString(buffer, offset, count);
            
            //异步调用过滤器进行处理
            ThreadPool.QueueUserWorkItem(new WaitCallback((sender) =>
            {
                //提交至过滤器处理
                filterHandler.Filter(sender as FilterContext);
            }), filterContext);
            return 0;
        }

        #region 接口实现
        public override bool CanRead
        {
            get { return output.CanRead; }
        }

        public override bool CanSeek
        {
            get { return output.CanSeek; }
        }

        public override bool CanWrite
        {
            get { return output.CanWrite; }
        }

        public override void Flush()
        {
            output.Flush();
        }

        public override long Length
        {
            get { return output.Length; }
        }

        public override long Position
        {
            get { return output.Position; }
            set { output.Position = value; }
        }

        public override int Read(byte[] buffer, int offset, int count)
        {
            return output.Read(buffer, offset, count);
        }

        public override long Seek(long offset, SeekOrigin origin)
        {
            return output.Seek(offset, origin);
        }

        public override void SetLength(long value)
        {
            output.SetLength(value);
        }

        public override void Write(byte[] buffer, int offset, int count)
        {
            try
            {
                var code = Filter(buffer, offset, count);
                if (code != 0)
                {
                    FilterLog.Log.DebugFormat("Filter执行返回值:{0}", code);
                }
            }
            catch (Exception ex)
            {
                FilterLog.Log.Debug("调用StreamFilter方法出错", ex);
            }


            output.Write(buffer, offset, count);
        }
        #endregion
    }
}