﻿using System;
using Core.AssemblyType;

namespace Core.Module.HttpFilter
{
    /// <summary>
    /// Filter 工厂，负责创建及调用Filter
    /// </summary>
    public class HttpFilterFactory
    {
        static HttpFilterFactory()
        {
            _staticFilterTypeCache.EnsureInitialized(new BuildManagerWrapper());
        }

        private static FilterTypeCache _staticFilterTypeCache = new FilterTypeCache();

        public void Filter(FilterContext context)
        {
            FilterLog.Log.Debug("进入FilterFactory处理");
            var filterActions = _staticFilterTypeCache.GetFilterAction(context.Controller, context.Action);
            if (null == filterActions || filterActions.Count == 0)
            {
                FilterLog.Log.WarnFormat("未找到关联的Filter处理 {0}/{1}", context.Controller, context.Action);
                return;
            }
            foreach (var action in filterActions)
            {
                try
                {
                    action(context);
                }
                catch (Exception ex)
                {
                    FilterLog.Log.ErrorFormat("{0},{1},{2},{3}", context.Controller, context.Action, context.UserName, context.IP);
                    FilterLog.Log.Error("Filter挂载执行错误", ex);
                }
            }
        }

        public bool HasFilter(FilterContext context)
        {
            return _staticFilterTypeCache.HasFilterAction(context.Controller, context.Action);
        }
    }
}