﻿using System;
using System.Collections.Specialized;
using System.Diagnostics;
using System.IO;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.SessionState;
namespace Core.Module.HttpFilter
{
    public class HttpFilterModule : IHttpModule, IRequiresSessionState
    {
        HttpFilterFactory filterHandler = new HttpFilterFactory();

        static HttpFilterModule()
        {
            FilterLog.Log.Debug("模块已经加载");
        }

        public void Init(HttpApplication context)
        {
            //AcquireRequestState
            context.AcquireRequestState += new EventHandler((sender, e) =>
            {
                var app = sender as HttpApplication;

                //加载控制器适配器 拦截所有合法的控制器请求
                ControllerAdapter(app);
            });
        }

        public void Dispose()
        {
        }

        public void ControllerAdapter(HttpApplication app)
        {
            //定义初始参数
            var response = app.Response;
            var request = app.Request;
            var url = request.RawUrl == "/" ? "/Home/Index" : request.RawUrl;

            //没有url的报文不拦截
            if (0 >= url.Length)
            {
                FilterLog.Log.Debug("Url为空");
                return;
            }

            //检查url是否包含后缀名,包括后缀名的肯定不是路由地址
            var ext = Path.GetExtension(url);
            if (0 < ext.Length)
            {
                FilterLog.Log.DebugFormat("该Url不是控制器 url:{0} ", url);
                return;
            }

            //分析出Controller和Action
            var matches = Regex.Matches(url, @"/(?<controller>\w+)[^/]*/(?<action>\w+)/?\w*");
            if (1 != matches.Count)
            {
                FilterLog.Log.DebugFormat("未能匹配到控制器,路由地址:{0},匹配数:{1}", url, matches.Count);
                return;
            }

            //验证用户名 写法为了读取extjs存储Cookie
            //var userCookie = request.Cookies["ys-username"];
            //if (null == userCookie)
            //{
            //    FilterLog.Log.Debug("未获取到用户名Cookie");
            //    return;
            //}
            //var ye =  userCookie.Value.Split('_');
            //if (2 != ye.Length)
            //{
            //    FilterLog.Log.Debug("用户名Cookie值格式不正确");
            //    return;
            //}
            //if (string.IsNullOrEmpty(ye[1]))
            //{
            //    FilterLog.Log.Debug("用户名为空");
            //    return;
            //}
            //调试需要
            var ye = new string[] { "2", "2" };

            //根据路由器名称，将信息提交至对应的日志处理模块
            var fc = new FilterContext();
            fc.Controller = matches[0].Groups["controller"].Value;
            fc.Action = matches[0].Groups["action"].Value;
            fc.UserName = ye[1];
            fc.IP = request.UserHostAddress == "::1" ? "127.0.0.1" : request.UserHostAddress;

            //首先判断是否存在关联的Filter处理
            if (!filterHandler.HasFilter(fc))
            {
                FilterLog.Log.DebugFormat("未找到关联的Filter Controller,控制器:{0} Action", fc.Controller, fc.Action);
                return;
            }

            //构造请求上下文（无论是get还是post都允许携带Url参数，post时允许携带body
            fc.InputParameter = new NameValueCollection(request.QueryString);
            if (app.Request.HttpMethod == "POST")
            {
                fc.InputParameter.Add(request.Form);
            }

            app.Response.Filter = new HttpFileterStream(app, filterHandler, fc);
        }
    }
}